<?php
/**
 *	This file is part of Store Manager Connector.
 *
 *   Store Manager Connector is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   Store Manager Connector is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Mobile Assistant Connector. If not, see <http://www.gnu.org/licenses/>.
 *
 *  @author    eMagicOne <contact@emagicone.com>
 *  @copyright 2022 eMagicOne
 *  @license   http://www.gnu.org/licenses   GNU General Public License
 */

if (!defined('ABSPATH')) {
    exit;
}

define('EMO_SMC_MODULE_NAME', 'store-manager-connector');
define('EMO_SMC_OPTIONS_NAME', 'SMCONNECTOR_OPTIONS');

require_once plugin_dir_path(__FILE__) . '/classes/smconnector_common.php';

class EmoSMCWoocommerceOverrider extends EmoSMConnectorCore
{
    public function __construct($module_name, $options_name)
    {
        $this->module_name = $module_name;
        $this->options_name = $options_name;
    }

    private static function getBaseUploadDirectory()
    {
        $data = wp_get_upload_dir();

        return $data['basedir'];
    }

    private static function getImageSizes()
    {
        global $_wp_additional_image_sizes;

        $sizes = array();

        foreach ( get_intermediate_image_sizes() as $s ) {
            $sizes[$s] = array( 'width' => '', 'height' => '', 'crop' => false );

            if (isset($_wp_additional_image_sizes[$s]['width'])) {
                $sizes[$s]['width'] = (int)$_wp_additional_image_sizes[$s]['width']; // For theme-added sizes
            } else {
                $sizes[$s]['width'] = get_option("{$s}_size_w"); // For default sizes set in options
            }

            if (isset($_wp_additional_image_sizes[$s]['height'])) {
                $sizes[$s]['height'] = (int)$_wp_additional_image_sizes[$s]['height']; // For theme-added sizes
            } else {
                $sizes[$s]['height'] = get_option("{$s}_size_h"); // For default sizes set in options
            }

            if (isset($_wp_additional_image_sizes[$s]['crop'])) {
                $sizes[$s]['crop'] = $_wp_additional_image_sizes[$s]['crop']; // For theme-added sizes
            } else {
                $sizes[$s]['crop'] = get_option("{$s}_crop"); // For default sizes set in options
            }
        }

        return $sizes;
    }

    public function isModuleEnabled()
    {
        include_once str_replace(site_url() . '/', ABSPATH, admin_url('includes/plugin.php'));

        if (is_plugin_active('store-manager-connector/smconnector.php')
            || in_array(
                'store-manager-connector/smconnector.php',
                apply_filters('active_plugins', get_option('active_plugins')),
                true
            )
        ) {
            $plugin_status = true;
        }

        return $plugin_status;
    }

    public function getSMConnectorOptions()
    {
        return get_option($this->options_name);
    }

    public function getShopRootDir()
    {
        return $this->subStr(ABSPATH, -1) == '/' ? ABSPATH : ABSPATH . '/';
    }

    public function getDbHost()
    {
        global $wpdb;

        return $wpdb->dbhost;
    }

    public function getDbName()
    {
        global $wpdb;

        return $wpdb->dbname;
    }

    public function getDbUsername()
    {
        global $wpdb;

        return $wpdb->dbuser;
    }

    public function getDbPassword()
    {
        global $wpdb;

        return $wpdb->dbpassword;
    }

    public function getDbPrefix()
    {
        global $wpdb;

        return $wpdb->prefix;
    }

    public function getSqlResults($sql, $type = self::ASSOC)
    {
        global $wpdb;

        try {
            if ($type == self::ASSOC) {
                $result = $wpdb->get_results($sql, ARRAY_A);
            } else {
                $result = $wpdb->get_results($sql, ARRAY_N);
            }

            if ($wpdb->last_error) {
                throw new Exception('Error');
            }
        } catch (Exception $e) {
            $this->error_no = @mysqli_errno($wpdb->dbh);
            $this->error_msg = $wpdb->last_error;

            return false;
        }

        return $result;
    }

    public function execSql($sql, $reconnect = false)
    {
        global $wpdb;

        $result = true;

        if (!$this->is_sql_encoded_in_base64) {
            $sql = $this->stripSlashes($this->stripSlashes($sql));
        }

        if ($reconnect) {
            $wpdb->db_connect();
        }

        try {
            if (false === $wpdb->query($sql)) {
                throw new Exception('Error');
            }
        } catch (Exception $e) {
            $this->error_no = @mysqli_errno($wpdb->dbh);
            $this->error_msg = $wpdb->last_error;
            $result = false;
        }

        return $result;
    }

    public function sanitizeSql($sql)
    {
        global $wpdb;

        $sql = $wpdb->_real_escape($sql);

        if (method_exists($wpdb, 'remove_placeholder_escape')) {
            $sql = $wpdb->remove_placeholder_escape($sql);
        }

        return $sql;
    }

    public function issetRequestParam($param)
    {
        return isset($_REQUEST[$param]);
    }

    public function getRequestParam($param)
    {
        return isset($_REQUEST[$param]) ? wp_kses($_REQUEST[$param], 'entities') : '';
    }

    public function getStoreLink($ssl = false)
    {
        return get_site_url();
    }

    public function runIndexer()
    {
        return false;
    }

    public function getCartVersion()
    {
        global $woocommerce;

        return $this->jsonEncode(array('cart_version' => $woocommerce->version));
    }

    public function getImage($entity_type, $image_id)
    {
        $upload_dir = wp_upload_dir();
        $image_path = preg_replace('#/+#', '/', $upload_dir['basedir'].'/' . $image_id);
        return $image_path;
    }

    public function setImage($entity_type, $image_id, $img, $type)
    {
        // Make thumbnails and other intermediate sizes.
        global $_wp_additional_image_sizes;

        $image_id = ltrim($image_id, "\\/");
        $img_file = self::getBaseUploadDirectory() . "/$image_id";
        $dir_name = dirname($img_file);

        if (!is_dir($dir_name) && !mkdir($dir_name, 0777, true)) {
            die(
                $this->jsonEncode(
                    array(
                        self::CODE_RESPONSE => self::ERROR_CODE_COMMON,
                        self::KEY_MESSAGE   => 'Could not create directory',
                    )
                )
            );
        }

        if ($type == self::IMAGE_URL) {
            $is_content_written = file_put_contents($img_file, $this->fileGetContents($img));
        } else {
            $is_content_written = move_uploaded_file(sanitize_textarea_field($_FILES[$img]['tmp_name']), $img_file);
        }

        if (!$is_content_written) {
            die(
                $this->jsonEncode(
                    array(
                        self::CODE_RESPONSE => self::ERROR_CODE_COMMON,
                        self::KEY_MESSAGE   => 'Could not create file',
                    )
                )
            );
        }

        $res                = false;
        $metadata           = array();
        $imagesize          = getimagesize($img_file);
        $metadata['width']  = $imagesize[0];
        $metadata['height'] = $imagesize[1];

        // Make the file path relative to the upload dir.
        $metadata['file'] = _wp_relative_upload_path($img_file);

        $sizes = apply_filters('intermediate_image_sizes_advanced', self::getImageSizes(), $metadata);

        if ($sizes) {
            $editor = wp_get_image_editor($img_file);

            if (!is_wp_error($editor)) {
                $res = $editor->multi_resize($sizes);
            }
        }

        if (!$res) {
            die(
                $this->jsonEncode(
                    array(
                    self::CODE_RESPONSE => self::ERROR_CODE_COMMON,
                    self::KEY_MESSAGE   => 'Unable to resize one or more of your pictures',
                    )
                )
            );
        }

        die(
            $this->jsonEncode(
                array(
                    self::CODE_RESPONSE => self::SUCCESSFUL,
                    self::KEY_MESSAGE   => 'Upload and resize of images has been executed successfully',
                )
            )
        );
    }

    public function getImageDir($type)
    {
        return self::getBaseUploadDirectory();
    }

    public function deleteImage($entity_type, $image_id)
    {
        $image_id = ltrim($image_id, "\\/");
        $this->deleteFile(self::getBaseUploadDirectory() . "/$image_id");
    }

    public function deleteFile($filepath)
    {
        if (!file_exists($filepath)) {
            die(
                $this->jsonEncode(
                    array(
                        self::CODE_RESPONSE => self::ERROR_CODE_COMMON,
                        self::KEY_MESSAGE   => 'File is missing on server',
                    )
                )
            );
        }

        if (unlink($filepath)) {
            die(
                $this->jsonEncode(
                    array(
                        self::CODE_RESPONSE => self::SUCCESSFUL,
                        self::KEY_MESSAGE   => 'File was deleted from FTP Server successfully',
                    )
                )
            );
        }

        die(
            $this->jsonEncode(
                array(
                    self::CODE_RESPONSE => self::ERROR_CODE_COMMON,
                    self::KEY_MESSAGE   => 'File was not deleted from FTP Server',
                )
            )
        );
    }

    public function copyImage($entity_type, $from_image_id, $to_image_id)
    {
        return false;
    }

    public function getFile($folder, $filename)
    {
        $folder   = trim($folder, '/');
        $filename = ltrim($filename, '/');
        if (empty($folder)) {
            return $this->getShopRootDir() . $filename;
        }

        return $this->getShopRootDir() . "$folder/$filename";
    }

    public function setFile($folder, $filename, $file)
    {
        $folder           = trim($folder, "\\/");
        $filename         = ltrim($filename, "\\/");
        $destination_path = $this->getShopRootDir() . "$folder/$filename";
        $dir_name         = dirname($destination_path);

        if (!is_dir($dir_name) && !mkdir($dir_name, 0777, true)) {
            die(
                $this->jsonEncode(
                    array(
                        self::CODE_RESPONSE => self::ERROR_CODE_COMMON,
                        self::KEY_MESSAGE   => 'Could not create directory',
                    )
                )
            );
        }

        $result = move_uploaded_file($_FILES[$file]['tmp_name'], $destination_path);

        if ($result) {
            die(
                $this->jsonEncode(
                    array(
                        self::CODE_RESPONSE => self::SUCCESSFUL,
                        self::KEY_MESSAGE   => 'File was successfully uploaded',
                    )
                )
            );
        } else {
            die(
                $this->jsonEncode(
                    array(
                        self::CODE_RESPONSE => self::ERROR_CODE_COMMON,
                        self::KEY_MESSAGE   => 'File was not uploaded',
                    )
                )
            );
        }
    }

    public function checkDataChanges($tables_arr = array())
    {
        if (!$tables_arr) {
            die(
                $this->jsonEncode(
                    array(
                        self::CODE_RESPONSE => self::ERROR_CODE_COMMON,
                        self::KEY_MESSAGE   => 'Parameter tables is empty',
                    )
                )
            );
        }

        $arr_result = array();

        foreach ($tables_arr as $table) {
            $table = trim($table);

            if (empty($table)) {
                continue;
            }

            try {
                $query = "SELECT `AUTO_INCREMENT` AS 'auto_increment'
                    FROM INFORMATION_SCHEMA.TABLES
                    WHERE TABLE_SCHEMA = '" . $this->getDbName() . "'
                    AND TABLE_NAME = '$table'";
                $result = $this->getSqlResults($query);

                if ($result === false) {
                    continue;
                }

                if (null !== $result[0]['auto_increment']) {
                    $arr_result[$table] = $result[0]['auto_increment'] - 1;
                } else {
                    $query = "SELECT `COLUMN_NAME` AS 'primary_key' INTO @primary_key_field
                        FROM `information_schema`.`COLUMNS`
                        WHERE (`TABLE_SCHEMA` = '" . $this->getDbName() . "') AND (`TABLE_NAME` = '$table')
                        AND (`COLUMN_KEY` = 'PRI')";
                    $this->execSql($query);
                    $query = "SET @s = CONCAT('SELECT MAX(', @primary_key_field, ') AS max_id FROM $table')";
                    $this->execSql($query);
                    $query = "PREPARE stmt FROM @s";
                    $this->execSql($query);
                    $query = "EXECUTE stmt;";
                    $result = $this->execSql($query);
                    $arr_result[$table] = $result[0]['max_id'];
                }
            } catch (Exception $e) {
                die(
                    $this->jsonEncode(
                        array(
                            self::CODE_RESPONSE => self::ERROR_CODE_COMMON,
                            self::KEY_MESSAGE   => $e->getMessage(),
                        )
                    )
                );
            }
        }

        die(
            $this->jsonEncode(
                array(
                    self::CODE_RESPONSE => self::SUCCESSFUL,
                    self::KEY_MESSAGE   => $arr_result,
                )
            )
        );
    }

    public function getNewOrders($order_id = '')
    {
        $order_id = (int)$order_id;

        if ($order_id < 1) {
            die(
                $this->jsonEncode(
                    array(
                        self::CODE_RESPONSE => self::ERROR_CODE_COMMON,
                        self::KEY_MESSAGE   => 'Order ID is incorrect',
                    )
                )
            );
        }

        $orders = array();

        try {
            // Select new order count
            $query = "SELECT COUNT(ID) AS CountNewOrder FROM " . $this->getDbPrefix()
                . "posts WHERE ID > $order_id AND post_type = 'shop_order'";
            $result = $this->getSqlResults($query);
            if (!$result) {
                die(
                    $this->jsonEncode(
                        array(
                            self::CODE_RESPONSE => self::ERROR_CODE_COMMON,
                            self::KEY_MESSAGE   => 'Cannot get count of new orders',
                        )
                    )
                );
            }
            $count_new_orders = $result[0]['CountNewOrder'];

            // Select maximum order id
            $query = "SELECT  MAX(ID) AS MaxOrderId FROM " . $this->getDbPrefix()
                . "posts WHERE post_type = 'shop_order'";
            $result = $this->getSqlResults($query);
            if (!$result) {
                die(
                    $this->jsonEncode(
                        array(
                            self::CODE_RESPONSE => self::ERROR_CODE_COMMON,
                            self::KEY_MESSAGE   => 'Cannot get max order id',
                        )
                    )
                );
            }
            $max_order_id = $result[0]['MaxOrderId'];

            // Select new orders
            $query = 'SELECT posts.ID,
                    cusomer_user_id.meta_value AS customer_id,
                    order_total.meta_value AS order_total,
                    order_currency.meta_value AS order_currency_code,
                    first_name.meta_value AS firstname,
                    last_name.meta_value AS lastname
                FROM ' . $this->getDbPrefix() . 'posts AS posts
                LEFT JOIN ' . $this->getDbPrefix() . "postmeta AS cusomer_user_id ON posts.ID = cusomer_user_id.post_id
                    AND cusomer_user_id.meta_key = '_customer_user'
                LEFT JOIN " . $this->getDbPrefix() . "postmeta AS order_total ON posts.ID = order_total.post_id
                    AND order_total.meta_key = '_order_total'
                LEFT JOIN " . $this->getDbPrefix() . "postmeta AS order_currency ON posts.ID = order_currency.post_id
                    AND order_currency.meta_key = '_order_currency'
                LEFT JOIN " . $this->getDbPrefix() . "usermeta AS first_name ON cusomer_user_id.meta_value = first_name.user_id
                    AND first_name.meta_key = 'first_name'
                LEFT JOIN " . $this->getDbPrefix() . "usermeta AS last_name ON cusomer_user_id.meta_value = last_name.user_id
                    AND last_name.meta_key = 'last_name'
                WHERE posts.post_type = 'shop_order'";
            $result = $this->getSqlResults($query);
            if (!$result) {
                die(
                    $this->jsonEncode(
                        array(
                            self::CODE_RESPONSE => self::ERROR_CODE_COMMON,
                            self::KEY_MESSAGE   => 'Cannot get new orders',
                        )
                    )
                );
            }

            foreach ($result as $order) {
                $order['firstname'] = (string)$order['firstname'];
                $order['lastname']  = (string)$order['lastname'];
                $orders[]           = $order;
            }
        } catch (Exception $e) {
            die(
                $this->jsonEncode(
                    array(
                        self::CODE_RESPONSE => self::ERROR_CODE_COMMON,
                        self::KEY_MESSAGE   => $e->getMessage(),
                    )
                )
            );
        }

        die(
            $this->jsonEncode(
                array(
                    self::CODE_RESPONSE => self::SUCCESSFUL,
                    self::KEY_MESSAGE   => array(
                        'CountNewOrder' => $count_new_orders,
                        'MaxOrderId'    => $max_order_id,
                        'OrderInfo'     => $orders
                    ),
                )
            )
        );
    }

    public function clearCache()
    {
        return false;
    }

    public function strLen($str)
    {
        return strlen($str);
    }

    public function subStr($str, $start, $length = false)
    {
        return $length ? substr($str, $start, $length) : substr($str, $start);
    }

    public function strToLower($str)
    {
        return strtolower($str);
    }

    public function strToUpper($str)
    {
        return strtoupper($str);
    }

    public function jsonEncode($arr)
    {
        return json_encode($arr);
    }

    public function stripSlashes($str)
    {
        return stripslashes($str);
    }

    public function fileGetContents($file)
    {
        return file_get_contents($file);
    }

    public function filePutContents($path, $content, $mode = null)
    {
        return file_put_contents($path, $content, $mode);
    }

    public function pSQL($data)
    {
        return $data;
    }

    public function saveConfigData($data)
    {
        add_option('SMCONNECTOR_OPTIONS', serialize($data));
    }

    public function fileOpen($path, $mode)
    {
        return fopen($path, $mode);
    }

    public function fileClose($resource)
    {
        return fclose($resource);
    }

    public function isReadable($path)
    {
        return is_readable($path);
    }

    public function isWritable($path)
    {
        return is_writable($path);
    }

    public function isDirectory($path)
    {
        return is_dir($path);
    }

    public function isFile($path)
    {
        return is_file($path);
    }

    public function stat($path)
    {
        return stat($path);
    }

    public function search($path, $pattern = '*', $onlyDir = false)
    {
        $data = glob("$path/$pattern");

        if ($onlyDir) {
            $dirs = array();

            foreach ($data as $item) {
                if ($this->isDirectory($item)) {
                    $dirs[] = $item;
                }
            }

            return $dirs;
        }

        return $data;
    }

    public function readDirectory($path)
    {
        return readdir($path);
    }

    public function filemtime($path)
    {
        return filemtime($path);
    }

    public function fileSize($path)
    {
        return filesize($path);
    }

    public function fileExists($path)
    {
        return file_exists($path);
    }

    public function fileWrite($resource, $data)
    {
        return fwrite($resource, $data);
    }

    public function fileRead($resource, $length)
    {
        return fread($resource, $length);
    }

    public function gzFileOpen($path, $mode)
    {
        return gzopen($path, $mode);
    }

    public function gzFileWrite($resource, $data)
    {
        gzwrite($resource, $data);
    }

    public function gzFileClose($resource)
    {
        gzclose($resource);
    }

    public function unlink($path)
    {
        return unlink($path);
    }

    public function createDirectory($path, $permissions)
    {
        return mkdir($path, $permissions);
    }

    public function getParentDirectory($path)
    {
        return dirname($path);
    }

    public function unserialize($data)
    {
        return unserialize($data);
    }

    public function getPaymentAndShippingMethods()
    {
        $shippingMethods = array();
        $paymentMethods  = array();

        // Get shipping methods
        $wcShiping = WC_Shipping::instance();
        $methods   = $wcShiping->load_shipping_methods();
        foreach ($methods as $method) {
            $shippingMethod = array('id' => $method->id);

            if (!empty($method->title)) {
                $shippingMethod['title'] = $method->title;
            }

            if (!empty($method->method_title)) {
                $shippingMethod['method_title'] = $method->method_title;
            }

            $shippingMethods[] = $shippingMethod;
        }

        // Get payment methods
        $wcPayment = WC_Payment_Gateways::instance();
        $methods   = $wcPayment->get_available_payment_gateways();
        foreach ($methods as $method) {
            $paymentMethod = array('id' => $method->id);

            if (!empty($method->title)) {
                $paymentMethod['title'] = $method->title;
            }

            if (!empty($method->method_title)) {
                $paymentMethod['method_title'] = $method->method_title;
            }

            $paymentMethods[] = $paymentMethod;
        }

        die(
            $this->jsonEncode(
                array(
                    self::CODE_RESPONSE => self::SUCCESSFUL,
                    self::KEY_MESSAGE   => array('shipping_methods' => $shippingMethods, 'payment_methods' => $paymentMethods)
                )
            )
        );
    }

    public function getZipArchiveInstance()
    {
        return new ZipArchive();
    }

    public function getZipArchiveCreateValue()
    {
        return ZipArchive::CREATE;
    }
}

class EmoStoreManagerConnector
{
    const MODULE_VERSION        = '1.1.1';
    const SMCONNECTOR_VERSION   = 11;

    private $shop_cart_overrider;

    public function __construct()
    {
        $this->shop_cart_overrider = new EmoSMCWoocommerceOverrider(EMO_SMC_MODULE_NAME, EMO_SMC_OPTIONS_NAME);
        new EmoSMConnectorCommon($this->shop_cart_overrider, self::MODULE_VERSION, self::SMCONNECTOR_VERSION);
    }
}
